@@ -69,7 +69,7 @@ from privacyidea.api.lib.prepolicy import (is_remote_user_allowed, prepolicy,
                                            webauthntoken_auth)
 from privacyidea.api.lib.utils import (send_result, get_all_params,
                                        verify_auth_token, getParam)
-from privacyidea.lib.utils import get_client_ip, hexlify_and_unicode
+from privacyidea.lib.utils import get_client_ip, hexlify_and_unicode, to_unicode
 from privacyidea.lib.config import get_from_config, SYSCONF, ensure_no_config_object
 from privacyidea.lib.event import event, EventConfiguration
 from privacyidea.lib import _
@@ -309,9 +309,11 @@ def get_auth_token():
                                                     superuser_realms=
                                                     superuser_realms)
         details = details or {}
-        if db_admin_exist(loginname) and realm == get_default_realm():
+        if db_admin_exist(loginname) and user_auth and realm == get_default_realm():
             # If there is a local admin with the same login name as the user
             # in the default realm, we inform about this in the log file.
+            # This condition can only be checked if the user was authenticated as it
+            # is the only way to verify if such a user exists.
             log.warning("A user '{0!s}' exists as local admin and as user in "
                         "your default realm!".format(loginname))
         if role == ROLE.ADMIN:
@@ -367,11 +369,11 @@ def get_auth_token():
                         "authtype": authtype,
                         "exp": datetime.utcnow() + validity,
                         "rights": rights},
-                       secret, algorithm='HS256').decode('utf8')
+                       secret, algorithm='HS256')
 
     # Add the role to the response, so that the WebUI can make decisions
     # based on this (only show selfservice, not the admin part)
-    return send_result({"token": token,
+    return send_result({"token": to_unicode(token),
                         "role": role,
                         "username": loginname,
                         "realm": realm,
