@@ -66,40 +66,43 @@ The functions of this module are tested in tests/test_api_lib_policy.py
 """
 
 import logging
-import string
 
 from OpenSSL import crypto
 
-from privacyidea.lib.error import PolicyError, RegistrationError, TokenAdminError
+from privacyidea.lib.error import PolicyError, RegistrationError, TokenAdminError, ResourceNotFoundError
 from flask import g, current_app
-from privacyidea.lib.policy import SCOPE, ACTION, PolicyClass
-from privacyidea.lib.policy import Match
+from privacyidea.lib.policy import SCOPE, ACTION, REMOTE_USER
+from privacyidea.lib.policy import Match, check_pin
 from privacyidea.lib.user import (get_user_from_param, get_default_realm,
                                   split_user, User)
-from privacyidea.lib.token import (get_tokens, get_realms_of_token, get_token_type)
-from privacyidea.lib.utils import (get_client_ip,
-                                   parse_timedelta, is_true, check_pin_policy, get_module_class,
-                                   determine_logged_in_userparams)
+from privacyidea.lib.token import (get_tokens, get_realms_of_token, get_token_type, get_token_owner)
+from privacyidea.lib.utils import (parse_timedelta, is_true, generate_charlists_from_pin_policy,
+                                   check_pin_contents, get_module_class,
+                                   determine_logged_in_userparams, parse_string_to_dict)
 from privacyidea.lib.crypto import generate_password
 from privacyidea.lib.auth import ROLE
 from privacyidea.api.lib.utils import getParam, attestation_certificate_allowed, is_fqdn
 from privacyidea.lib.clientapplication import save_clientapplication
-from privacyidea.lib.config import (get_token_class, get_from_config, SYSCONF)
+from privacyidea.lib.config import (get_token_class)
 import functools
 import jwt
 import re
 import importlib
 
 # Token specific imports!
-from privacyidea.lib.tokens.webauthn import (WebAuthnRegistrationResponse, AUTHENTICATOR_ATTACHMENT_TYPES,
-                                             USER_VERIFICATION_LEVELS, ATTESTATION_LEVELS, ATTESTATION_FORMS)
-from privacyidea.lib.tokens.webauthntoken import (WEBAUTHNACTION, DEFAULT_PUBLIC_KEY_CREDENTIAL_ALGORITHM_PREFERENCE,
+from privacyidea.lib.tokens.webauthn import (WebAuthnRegistrationResponse,
+                                             AUTHENTICATOR_ATTACHMENT_TYPES,
+                                             USER_VERIFICATION_LEVELS, ATTESTATION_LEVELS,
+                                             ATTESTATION_FORMS)
+from privacyidea.lib.tokens.webauthntoken import (WEBAUTHNACTION,
+                                                  DEFAULT_PUBLIC_KEY_CREDENTIAL_ALGORITHM_PREFERENCE,
                                                   PUBLIC_KEY_CREDENTIAL_ALGORITHM_PREFERENCE_OPTIONS,
                                                   DEFAULT_TIMEOUT, DEFAULT_ALLOWED_TRANSPORTS,
                                                   DEFAULT_USER_VERIFICATION_REQUIREMENT,
                                                   DEFAULT_AUTHENTICATOR_ATTESTATION_LEVEL,
-                                                  DEFAULT_AUTHENTICATOR_ATTESTATION_FORM, WebAuthnTokenClass,
-                                                  DEFAULT_CHALLENGE_TEXT_AUTH, DEFAULT_CHALLENGE_TEXT_ENROLL,
+                                                  DEFAULT_AUTHENTICATOR_ATTESTATION_FORM,
+                                                  WebAuthnTokenClass, DEFAULT_CHALLENGE_TEXT_AUTH,
+                                                  DEFAULT_CHALLENGE_TEXT_ENROLL,
                                                   is_webauthn_assertion_response)
 from privacyidea.lib.tokens.u2ftoken import (U2FACTION, parse_registration_data)
 from privacyidea.lib.tokens.u2f import x509name_to_string
@@ -153,6 +156,22 @@ class prepolicy(object):
         return policy_wrapper
 
 
+def _generate_pin_from_policy(policy, size=6):
+    """
+    This helper function creates a string of allowed characters from the value of a pincontents policy.
+
+    :param policy: The policy that describes the allowed contents of the PIN (see check_pin_contents).
+    :param size: The desired length of the generated pin
+    :return: The generated PIN
+    """
+
+    charlists_dict = generate_charlists_from_pin_policy(policy)
+
+    pin = generate_password(size=size, characters=charlists_dict['base'],
+                      requirements=charlists_dict['requirements'])
+    return pin
+
+
 def set_random_pin(request=None, action=None):
     """
     This policy function is to be used as a decorator in the API setrandompin function
@@ -182,8 +201,23 @@ def set_random_pin(request=None, action=None):
         raise TokenAdminError("You need to specify a policy '{0!s}' in scope "
                               "{1!s}.".format(ACTION.OTPPINSETRANDOM, role))
     elif len(pin_pols) == 1:
-        log.debug("Creating random OTP PIN with length {0!s}".format(list(pin_pols)[0]))
-        request.all_data["pin"] = generate_password(size=int(list(pin_pols)[0]))
+        # check pin contents policy per token type, otherwise fall back
+        tokentype = get_token_type(request.all_data.get("serial"))
+        pol_contents = Match.admin_or_user(g, action="{0!s}_{1!s}".format(tokentype, ACTION.OTPPINCONTENTS),
+                                           user_obj=request.User).action_values(unique=True)
+        if not pol_contents:
+            pol_contents = Match.admin_or_user(g, action=ACTION.OTPPINCONTENTS,
+                                               user_obj=request.User).action_values(unique=True)
+
+        if len(pol_contents) == 1:
+            log.info("Creating random OTP PIN with length {0!s} "
+                      "matching the contents policy {1!s}".format(list(pin_pols)[0], list(pol_contents)[0]))
+            # generate a pin which matches the contents requirement
+            r = _generate_pin_from_policy(list(pol_contents)[0], size=int(list(pin_pols)[0]))
+            request.all_data["pin"] = r
+        else:
+            log.debug("Creating random OTP PIN with length {0!s}".format(list(pin_pols)[0]))
+            request.all_data["pin"] = generate_password(size=int(list(pin_pols)[0]))
 
     return True
 
@@ -194,8 +228,8 @@ def init_random_pin(request=None, action=None):
     If the policy is set accordingly it adds a random PIN to the
     request.all_data like.
 
-    It uses the policy SCOPE.ENROLL, ACTION.OTPPINRANDOM to set a random OTP
-    PIN during Token enrollment
+    It uses the policy SCOPE.ENROLL, ACTION.OTPPINRANDOM and ACTION.OTPPINCONTENTS
+    to set a random OTP PIN during Token enrollment
     """
     params = request.all_data
     user_object = get_user_from_param(params)
@@ -203,8 +237,23 @@ def init_random_pin(request=None, action=None):
     pin_pols = Match.user(g, scope=SCOPE.ENROLL, action=ACTION.OTPPINRANDOM,
                           user_object=user_object).action_values(unique=True)
     if len(pin_pols) == 1:
-        log.debug("Creating random OTP PIN with length {0!s}".format(list(pin_pols)[0]))
-        request.all_data["pin"] = generate_password(size=int(list(pin_pols)[0]))
+        # check pin contents policy per token type, otherwise fall back
+        tokentype = request.all_data.get("type", "hotp")
+        pol_contents = Match.admin_or_user(g, action="{0!s}_{1!s}".format(tokentype, ACTION.OTPPINCONTENTS),
+                                           user_obj=request.User).action_values(unique=True)
+        if not pol_contents:
+            pol_contents = Match.admin_or_user(g, action=ACTION.OTPPINCONTENTS,
+                                               user_obj=request.User).action_values(unique=True)
+
+        if len(pol_contents) == 1:
+            log.info("Creating random OTP PIN with length {0!s} "
+                      "matching the contents policy {1!s}".format(list(pin_pols)[0], list(pol_contents)[0]))
+            # generate a pin which matches the contents requirement
+            r = _generate_pin_from_policy(list(pol_contents)[0], size=int(list(pin_pols)[0]))
+            request.all_data["pin"] = r
+        else:
+            log.debug("Creating random OTP PIN with length {0!s}".format(list(pin_pols)[0]))
+            request.all_data["pin"] = generate_password(size=int(list(pin_pols)[0]))
 
         # handle the PIN
         handle_pols = Match.user(g, scope=SCOPE.ENROLL, action=ACTION.PINHANDLING,
@@ -286,40 +335,37 @@ def check_otp_pin(request=None, action=None):
             tokentype = tokensobject_list[0].token.tokentype
     # the default tokentype is still HOTP
     tokentype = tokentype or "hotp"
-    # get the policies for minimum length, maximum length and PIN contents
-    # first try to get a token specific policy - otherwise fall back to
-    # default policy
-    pol_minlen = Match.admin_or_user(g, action="{0!s}_{1!s}".format(tokentype, ACTION.OTPPINMINLEN),
-                                     user_obj=request.User).action_values(unique=True)
-    if not pol_minlen:
-        pol_minlen = Match.admin_or_user(g, action=ACTION.OTPPINMINLEN,
-                                         user_obj=request.User).action_values(unique=True)
-    pol_maxlen = Match.admin_or_user(g, action="{0!s}_{1!s}".format(tokentype, ACTION.OTPPINMAXLEN),
-                                     user_obj=request.User).action_values(unique=True)
-    if not pol_maxlen:
-        pol_maxlen = Match.admin_or_user(g, action=ACTION.OTPPINMAXLEN,
-                                         user_obj=request.User).action_values(unique=True)
-    pol_contents = Match.admin_or_user(g, action="{0!s}_{1!s}".format(tokentype, ACTION.OTPPINCONTENTS),
-                                       user_obj=request.User).action_values(unique=True)
-    if not pol_contents:
-        pol_contents = Match.admin_or_user(g, action=ACTION.OTPPINCONTENTS,
-                                           user_obj=request.User).action_values(unique=True)
+    check_pin(g, pin, tokentype, request.User)
+    return True
 
-    if len(pol_minlen) == 1 and len(pin) < int(list(pol_minlen)[0]):
-        # check the minimum length requirement
-        raise PolicyError("The minimum OTP PIN length is {0!s}".format(
-                          list(pol_minlen)[0]))
 
-    if len(pol_maxlen) == 1 and len(pin) > int(list(pol_maxlen)[0]):
-        # check the maximum length requirement
-        raise PolicyError("The maximum OTP PIN length is {0!s}".format(
-                          list(pol_maxlen)[0]))
+def check_application_tokentype(request=None, action=None):
+    """
+    This pre policy checks if the request is allowed to specify the tokentype.
+    If the policy is not set, a possibly set parameter "type" is removed
+    from the request.
 
-    if len(pol_contents) == 1:
-        # check the contents requirement
-        r, comment = check_pin_policy(pin, list(pol_contents)[0])
-        if r is False:
-            raise PolicyError(comment)
+    Check ACTION.APPLICATION_TOKENTYPE
+
+    This decorator should wrap
+        /validate/check, /validate/samlcheck and /validate/triggerchallenge.
+
+    :param request: The request that is intercepted during the API call
+    :type request: Request Object
+    :param action: An optional Action
+    :type action: basestring
+    :returns: Always true. Modified the parameter request
+    """
+    application_allowed = Match.generic(g, scope=SCOPE.AUTHZ,
+                                        action=ACTION.APPLICATION_TOKENTYPE,
+                                        user_object=request.User,
+                                        active=True).any()
+
+    # if the application is not allowed, we remove the tokentype
+    if not application_allowed and "type" in request.all_data:
+        log.info("Removing parameter 'type' from request, "
+                 "since application is not allowed to authenticate by token type.")
+        del request.all_data["type"]
 
     return True
 
@@ -460,6 +506,40 @@ def init_token_defaults(request=None, action=None):
     return True
 
 
+def init_registrationcode_length_contents(request=None, action=None):
+    """
+    This policy function is to be used as a decorator in the API token init function.
+
+    If there is a valid policy set the action values of REGISTRATIONCODE_LENGTH
+    and REGISTRATIONCODE_CONTENTS are added to request.all_data as
+
+    { 'registration.length': '10', 'registration.contents': 'cn' }
+    """
+    from privacyidea.lib.tokens.registrationtoken import DEFAULT_LENGTH, DEFAULT_CONTENTS
+    no_content_policy = True
+    no_length_policy = True
+    params = request.all_data
+    tokentype = params.get("type")
+    if tokentype == "registration":
+        user_object = get_user_from_param(params)
+        length_pols = Match.user(g, scope=SCOPE.ENROLL, action=ACTION.REGISTRATIONCODE_LENGTH,
+                                user_object=user_object).action_values(unique=True)
+        if len(length_pols) == 1:
+            request.all_data[ACTION.REGISTRATIONCODE_LENGTH] = list(length_pols)[0]
+            no_length_policy = False
+        content_pols = Match.user(g, scope=SCOPE.ENROLL, action=ACTION.REGISTRATIONCODE_CONTENTS,
+                                user_object=user_object).action_values(unique=True)
+        if len(content_pols) == 1:
+            request.all_data[ACTION.REGISTRATIONCODE_CONTENTS] = list(content_pols)[0]
+            no_content_policy = False
+    # if there is no policy, set defaults.
+    if no_length_policy:
+        request.all_data[ACTION.REGISTRATIONCODE_LENGTH] = DEFAULT_LENGTH
+    if no_content_policy:
+        request.all_data[ACTION.REGISTRATIONCODE_CONTENTS] = DEFAULT_CONTENTS
+    return True
+
+
 def init_tokenlabel(request=None, action=None):
     """
     This policy function is to be used as a decorator in the API init function.
@@ -561,7 +641,7 @@ def twostep_enrollment_parameters(request=None, action=None):
     to ``request.all_data``, that is:
 
      * ``{type}_2step_serversize`` is written to ``2step_serversize``
-     * ``{type}_2step_clientsize`` is written to ``2step_clientsize`
+     * ``{type}_2step_clientsize`` is written to ``2step_clientsize``
      * ``{type}_2step_difficulty`` is written to ``2step_difficulty``
 
     If no policy matches, the value passed by the user is kept.
@@ -616,9 +696,16 @@ def check_max_token_user(request=None, action=None):
     ERROR_ACTIVE = "The number of active tokens for this user is limited!"
     ERROR_ACTIVE_TYPE = "The number of active tokens of type {0!s} for this user is limited!"
     params = request.all_data
+    serial = getParam(params, "serial")
+    tokentype = getParam(params, "type")
     user_object = get_user_from_param(params)
+    if user_object.is_empty() and serial:
+        try:
+            user_object = get_token_owner(serial) or User()
+        except ResourceNotFoundError:
+            # in case of token init the token does not yet exist in the db
+            pass
     if user_object.login:
-        serial = getParam(params, "serial")
         tokentype = getParam(params, "type")
         if not tokentype:
             if serial:
@@ -811,6 +898,65 @@ def required_email(request=None, action=None):
     return True
 
 
+def check_custom_user_attributes(request=None, action=None):
+    """
+    This pre condition checks for the policies delete_custom_user_attributes and
+    set_custom_user_attributes, if the user or admin is allowed to set or deleted
+    the requested attribute.
+
+    It decorates POST /user/attribute and DELETE /user/attribute/...
+
+    :param request: The request that is intercepted during the API call
+    :type request: Request Object
+    :param action: An optional action, (would be set/delete)
+    :return: Raises a PolicyError, if the wrong attribute is given.
+    """
+    ERROR = "You are not allowed to {0!s} the custom user attribute {1!s}!"
+    is_allowed = False
+    if action == "delete":
+        attr_pol_dict = Match.admin_or_user(g, action=ACTION.DELETE_USER_ATTRIBUTES,
+                                            user_obj=request.User).action_values(unique=False,
+                                                                                 allow_white_space_in_action=True)
+        attr_key = request.all_data.get("attrkey")
+        for attr_pol_val in attr_pol_dict:
+            attr_pol_list = [x.strip() for x in attr_pol_val.strip().split() if x]
+            if attr_key in attr_pol_list or "*" in attr_pol_list:
+                is_allowed = True
+                break
+        if is_allowed:
+            g.audit_object.add_policy(attr_pol_dict.get(attr_pol_val))
+        else:
+            raise PolicyError(ERROR.format(action, attr_key))
+    elif action == "set":
+        attr_pol_dict = Match.admin_or_user(g, action=ACTION.SET_USER_ATTRIBUTES,
+                                            user_obj=request.User).action_values(unique=False,
+                                                                                 allow_white_space_in_action=True)
+        attr_key = request.all_data.get("key")
+        attr_value = request.all_data.get("value")
+        for pol_string in attr_pol_dict:
+            pol_dict = parse_string_to_dict(pol_string)
+            if attr_value in pol_dict.get(attr_key, []):
+                # It is allowed to set the key to this value
+                is_allowed = True
+                break
+            if attr_value in pol_dict.get("*", []):
+                # this value can be set in any key
+                is_allowed = True
+                break
+            if "*" in pol_dict.get(attr_key, []):
+                # This key can be set to any value
+                is_allowed = True
+                break
+            if "*" in pol_dict.get("*", []):
+                # Any key can be set to any value
+                is_allowed = True
+                break
+        if is_allowed:
+            g.audit_object.add_policy(attr_pol_dict.get(pol_string))
+        else:
+            raise PolicyError(ERROR.format(action, attr_key))
+
+
 def auditlog_age(request=None, action=None):
     """
     This pre condition checks for the policy auditlog_age and set the
@@ -845,6 +991,29 @@ def auditlog_age(request=None, action=None):
     return True
 
 
+def hide_audit_columns(request=None, action=None):
+    """
+    This pre condition checks for the policy hide_audit_columns and sets the
+    "hidden_columns" parameter for the audit search. The given columns will be
+    removed from the returned audit dict.
+
+    Check ACTION.HIDE_AUDIT_COLUMNS
+
+    The decorator should wrap GET /audit/
+
+    :param request: The request that is intercepted during the API call
+    :type request: Request Object
+    :param action: An optional Action
+    :type action: basestring
+    :returns: Always true. Modified the parameter request
+    """
+    hidden_columns = Match.admin_or_user(g, action=ACTION.HIDE_AUDIT_COLUMNS,
+                                         user_obj=request.User).action_values(unique=False)
+    request.all_data["hidden_columns"] = list(hidden_columns)
+
+    return True
+
+
 def mangle(request=None, action=None):
     """
     This pre condition checks if either of the parameters pass, user or realm
@@ -953,10 +1122,11 @@ def check_base_action(request=None, action=None, anonymous=False):
     """
     This decorator function takes the request and verifies the given action
     for the SCOPE ADMIN or USER.
+
     :param request:
     :param action:
     :param anonymous: If set to True, the user data is taken from the request
-        parameters.
+                      parameters.
     :return: True otherwise raises an Exception
     """
     ERROR = {"user": "User actions are defined, but the action %s is not "
@@ -1017,12 +1187,14 @@ def check_token_upload(request=None, action=None):
     upload_allowed = True
     if tokenrealms:
         for trealm in tokenrealms.split(","):
-            if not Match.generic(g, action=ACTION.IMPORT, adminuser=g.logged_in_user.get("username"),
-                                           adminrealm=g.logged_in_user.get("realm"), realm=trealm).allowed():
+            if not Match.generic(g, scope=SCOPE.ADMIN, action=ACTION.IMPORT,
+                                 adminuser=g.logged_in_user.get("username"),
+                                 adminrealm=g.logged_in_user.get("realm"), realm=trealm).allowed():
                 upload_allowed = False
     else:
-        upload_allowed = Match.generic(g, action=ACTION.IMPORT, adminuser=g.logged_in_user.get("username"),
-                                        adminrealm=g.logged_in_user.get("realm")).allowed()
+        upload_allowed = Match.generic(g, scope=SCOPE.ADMIN, action=ACTION.IMPORT,
+                                       adminuser=g.logged_in_user.get("username"),
+                                       adminrealm=g.logged_in_user.get("realm")).allowed()
     if not upload_allowed:
         raise PolicyError("Admin actions are defined, but you are not allowed to upload token files.")
     return True
@@ -1141,26 +1313,32 @@ def mock_fail(req, action):
     raise Exception("This is an Exception in an external check function")
 
 
-def is_remote_user_allowed(req):
+def is_remote_user_allowed(req, write_to_audit_log=True):
     """
     Checks if the REMOTE_USER server variable is allowed to be used.
 
     .. note:: This is not used as a decorator!
 
     :param req: The flask request, containing the remote user and the client IP
-    :return: a bool value
+    :param write_to_audit_log: whether the policy name should be added to the audit log entry
+    :type write_to_audit_log: bool
+    :return: Return a value or REMOTE_USER, can be "disable", "active" or "force".
+    :rtype: str
     """
-    res = False
     if req.remote_user:
         loginname, realm = split_user(req.remote_user)
         realm = realm or get_default_realm()
         ruser_active = Match.generic(g, scope=SCOPE.WEBUI,
                                      action=ACTION.REMOTE_USER,
                                      user=loginname,
-                                     realm=realm).action_values(unique=False)
-        res = bool(ruser_active)
+                                     realm=realm).action_values(unique=False,
+                                                                write_to_audit_log=write_to_audit_log)
+        # there should be only one action value here
+        if ruser_active:
+            return list(ruser_active)[0]
 
-    return res
+    # Return default "disable"
+    return REMOTE_USER.DISABLE
 
 
 def save_client_application_type(request, action):
@@ -1245,6 +1423,22 @@ def pushtoken_add_config(request, action):
         else:
             request.all_data[PUSH_ACTION.SSL_VERIFY] = "1"
 
+        # Get the TTL and the registration URL from the policies
+        registration_url = Match.user(g, scope=SCOPE.ENROLL, action=PUSH_ACTION.REGISTRATION_URL,
+                                      user_object=request.User if request.User else None) \
+            .action_values(unique=True, allow_white_space_in_action=True)
+        if len(registration_url) == 1:
+            request.all_data[PUSH_ACTION.REGISTRATION_URL] = list(registration_url)[0]
+        else:
+            raise PolicyError("Missing enrollment policy for push token: {0!s}".format(PUSH_ACTION.REGISTRATION_URL))
+        ttl = Match.user(g, scope=SCOPE.ENROLL, action=PUSH_ACTION.TTL,
+                         user_object=request.User if request.User else None) \
+            .action_values(unique=True, allow_white_space_in_action=True)
+        if len(ttl) == 1:
+            request.all_data[PUSH_ACTION.TTL] = list(ttl)[0]
+        else:
+            request.all_data[PUSH_ACTION.TTL] = "10"
+
 
 def u2ftoken_verify_cert(request, action):
     """
@@ -1376,7 +1570,7 @@ def indexedsecret_force_attribute(request, action):
 
     return True
 
-  
+
 def webauthntoken_request(request, action):
     """
     This is a WebAuthn token specific wrapper for all endpoints using WebAuthn tokens.
@@ -1514,7 +1708,7 @@ def webauthntoken_authz(request, action):
     :param action:
     :type action:
     :return:
-    :rtype
+    :rtype:
     """
 
     # If a WebAuthn token is being authorized.
@@ -1879,3 +2073,34 @@ def _attestation_certificate_allowed(attestation_cert, allowed_certs_pols):
         else None
 
     return attestation_certificate_allowed(cert_info, allowed_certs_pols)
+
+
+def required_piv_attestation(request, action=None):
+    """
+    This is a token specific decorator for certificate tokens for the endpoint
+    /token/init
+    According to the policy scope=SCOPE.ENROLL,
+    action=REQUIRE_ATTESTATION an exception is raised, if no attestation parameter is given.
+
+    It also checks the policy if the attestation should be verified and sets the
+    parameter verify_attestation accordingly.
+
+    :param request:
+    :param action:
+    :return:
+    """
+    from privacyidea.lib.tokens.certificatetoken import ACTION, REQUIRE_ACTIONS
+    ttype = request.all_data.get("type")
+    if ttype and ttype.lower() == "certificate":
+        # Get attestation certificate requirement
+        require_att = Match.user(g, scope=SCOPE.ENROLL, action=ACTION.REQUIRE_ATTESTATION,
+                                     user_object=request.User if request.User else None).action_values(unique=True)
+        if REQUIRE_ACTIONS.REQUIRE_AND_VERIFY in list(require_att):
+            if not request.all_data.get("attestation"):
+                # There is no attestation certificate in the request, although it is required!
+                log.warning("The request is missing an attestation certificate. {0!s}".format(require_att))
+                raise PolicyError("A policy requires that you provide an attestation certificate.")
+
+        # Add parameter verify_attestation
+        request.all_data["verify_attestation"] = REQUIRE_ACTIONS.VERIFY in list(require_att) or \
+                                                 REQUIRE_ACTIONS.REQUIRE_AND_VERIFY in list(require_att)
