@@ -33,6 +33,7 @@ It provides functions to retrieve (get) and and set configuration.
 The code is tested in tests/test_lib_config
 """
 
+import copy
 import sys
 import logging
 import inspect
@@ -44,6 +45,7 @@ from ..models import (Config, db, Resolver, Realm, PRIVACYIDEA_TIMESTAMP,
                       save_config_timestamp, Policy, EventHandler)
 from privacyidea.lib.framework import get_request_local_store, get_app_config_value, get_app_local_store
 from privacyidea.lib.utils import to_list
+from privacyidea.lib.utils.export import (register_import, register_export)
 from .crypto import encryptPassword
 from .crypto import decryptPassword
 from .resolvers.UserIdResolver import UserIdResolver
@@ -132,7 +134,8 @@ class SharedConfigClass(object):
                 for realm in Realm.query.all():
                     if realm.default:
                         default_realm = realm.name
-                    realmdef = {"option": realm.option,
+                    realmdef = {"id": realm.id,
+                                "option": realm.option,
                                 "default": realm.default,
                                 "resolver": []}
                     for x in realm.resolver_list:
@@ -209,7 +212,7 @@ class LocalConfigClass(object):
             "admin" or "public". If "public", only values with type="public"
             are returned.
         :type role: string
-        :param return_bool: If the a boolean value should be returned. Returns
+        :param return_bool: If a boolean value should be returned. Returns
             True if value is "True", "true", 1, "1", True...
         :return: If key is None, then a dictionary is returned. If a certain key
             is given a string/bool is returned.
@@ -224,7 +227,7 @@ class LocalConfigClass(object):
         for ckey, cvalue in self.config.items():
             if role == "admin" or cvalue.get("Type") == "public":
                 reduced_config[ckey] = self.config[ckey]
-        if not reduced_config and role=="admin":
+        if not reduced_config and role == "admin":
             reduced_config = self.config
 
         for ckey, cvalue in reduced_config.items():
@@ -855,7 +858,7 @@ def get_machine_resolver_module_list():
 def set_privacyidea_config(key, value, typ="", desc=""):
     """
     Set a config value and writes it to the Config database table.
-    Can by of type "password" or "public". "password" gets encrypted.
+    Can be of type "password" or "public". "password" gets encrypted.
     """
     if not typ:
         # check if this is a token specific config and if it should be public
@@ -866,7 +869,6 @@ def set_privacyidea_config(key, value, typ="", desc=""):
         except Exception:
             log.debug("This seems to be no token specific setting")
 
-    ret = 0
     if typ == "password":
         # store value in encrypted way
         value = encryptPassword(value)
@@ -978,3 +980,26 @@ def get_privacyidea_nodes():
         nodes.append(own_node_name)
     return nodes
 
+
+@register_export()
+def export_config():
+    """Export the global configuration"""
+    c = copy.copy(get_config_object().config)
+    c.pop('__timestamp__')
+    return c
+
+
+@register_import()
+def import_config(data):
+    """Import given server configuration"""
+    log.debug('Import server config: {0!s}'.format(data))
+    res = {}
+    for key, values in data.items():
+        r = set_privacyidea_config(key, values['Value'],
+                                   desc=values['Description'] if 'Description' in values else None,
+                                   typ=values['Type'] if 'Type' in values else None)
+        res[key] = r
+    log.info('Added configuration: {0!s}'.format(
+        ', '.join([k for k, v in res.items() if v == 'insert'])))
+    log.info('Updated configuration: {0!s}'.format(
+        ', '.join([k for k, v in res.items() if v == 'update'])))
