@@ -940,7 +940,7 @@ def gen_serial(tokentype=None, prefix=None):
 
 
 @log_with(log)
-def import_token(serial, token_dict, default_hashlib=None, tokenrealms=None):
+def import_token(serial, token_dict, tokenrealms=None):
     """
     This function is used during the import of a PSKC file.
 
@@ -959,8 +959,6 @@ def import_token(serial, token_dict, default_hashlib=None, tokenrealms=None):
             }
 
     :type token_dict: dict
-    :param default_hashlib: Set the given hashlib as default
-    :type default_hashlib: str
     :param tokenrealms: List of realms to set as realms of the token
     :type tokenrealms: list
     :return: the token object
@@ -978,9 +976,6 @@ def import_token(serial, token_dict, default_hashlib=None, tokenrealms=None):
                         token_dict.get("user").get("realm"),
                         token_dict.get("user").get("resolver"))
 
-    if default_hashlib and default_hashlib != "auto":
-        init_param['hashlib'] = default_hashlib
-
     # Imported tokens are usually hardware tokens
     token = init_token(init_param, user=user_obj,
                        tokenrealms=tokenrealms,
@@ -1081,8 +1076,7 @@ def init_token(param, user=None, tokenrealms=None,
     if user is not None and user.login != "":
         tokenobject.add_user(user)
 
-    upd_params = param
-    tokenobject.update(upd_params)
+    tokenobject.update(param)
 
     try:
         # Save the token to the database
@@ -2196,6 +2190,11 @@ def check_token_list(tokenobject_list, passw, user=None, options=None, allow_res
             raise TokenAdminError(_("This action is not possible, since the "
                                     "token is locked"), id=1007)
 
+    # Remove certain disabled tokens from tokenobject_list
+    if len(tokenobject_list) > 0:
+        tokenobject_list = [token for token in tokenobject_list if (token.is_active()
+                                                                    or token.check_if_disabled)]
+
     for tokenobject in sorted(tokenobject_list, key=weigh_token_type):
         if log.isEnabledFor(logging.DEBUG):
             # Avoid a SQL query triggered by ``tokenobject.user`` in case
