@@ -33,8 +33,7 @@ import binascii
 import base64
 import qrcode
 import sqlalchemy
-from six.moves.urllib.parse import urlunparse, urlparse, urlencode
-from six.moves import zip_longest
+from six.moves.urllib.parse import urlencode
 from io import BytesIO
 import string
 import re
@@ -46,9 +45,9 @@ from netaddr import IPAddress, IPNetwork, AddrFormatError
 import hashlib
 import traceback
 import threading
-import pkg_resources
+import importlib_metadata
 import time
-import cgi
+import html
 
 from privacyidea.lib.error import ParameterError, ResourceNotFoundError, PolicyError
 
@@ -56,8 +55,12 @@ ENCODING = "utf-8"
 
 BASE58 = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
 
-ALLOWED_SERIAL = "^[0-9a-zA-Z\-_]+$"
+ALLOWED_SERIAL = r"^[0-9a-zA-Z\-_]+$"
 
+# character lists for the identifiers in the pin content policy
+CHARLIST_CONTENTPOLICY = {"c": string.ascii_letters, # characters
+                          "n": string.digits,        # numbers
+                          "s": string.punctuation}   # special
 
 def check_time_in_range(time_range, check_time=None):
     """
@@ -388,7 +391,7 @@ def decode_base32check(encoded_data, always_upper=True):
     return hexlify_and_unicode(payload)
 
 
-def sanity_name_check(name, name_exp="^[A-Za-z0-9_\-\.]+$"):
+def sanity_name_check(name, name_exp=r"^[A-Za-z0-9_\-\.]+$"):
     """
     This function can be used to check the sanity of a name like a resolver,
     ca connector or realm.
@@ -441,8 +444,8 @@ def get_data_from_params(params, exclude_params, config_description, module,
                 if k in config_description:
                     types[k] = config_description.get(k)
                 else:
-                    log.warn("the passed key %r is not a "
-                             "parameter for the %s %r" % (k, module, type))
+                    log.warning("the passed key '{0!s}' is not a parameter for "
+                                "the {1!s} type '{2!s}'".format(k, module, type))
 
     # Check that there is no type or desc without the data itself.
     # i.e. if there is a type.BindPW=password, then there must be a
@@ -537,7 +540,7 @@ def parse_date(date_string):
         # If it stars with a year 2017/... we do NOT dayfirst.
         # See https://github.com/dateutil/dateutil/issues/457
         d = parse_date_string(date_string,
-                              dayfirst=re.match(r"^\d\d[/\.]", date_string))
+                              dayfirst=re.match(r"^\d\d[/.]", date_string))
     except ValueError:
         log.debug("Dateformat {0!s} could not be parsed".format(date_string))
 
@@ -610,8 +613,8 @@ def check_proxy(path_to_client, proxy_settings):
     try:
         proxy_dict = parse_proxy(proxy_settings)
     except AddrFormatError:
-        log.error("Error parsing the OverrideAuthorizationClient setting: {"
-                  "0!s}! The IP addresses need to be comma separated. Fix "
+        log.error("Error parsing the OverrideAuthorizationClient setting: "
+                  "{0!s}! The IP addresses need to be comma separated. Fix "
                   "this. The client IP will not be mapped!".format(proxy_settings))
         log.debug("{0!s}".format(traceback.format_exc()))
         return path_to_client[0]
@@ -822,23 +825,13 @@ def compare_condition(condition, value):
         return False
 
     try:
-        # compare equal
-        if condition[0] in "=" + string.digits:
-            if condition[0] == "=":
-                compare_value = int(condition[1:])
-            else:
-                compare_value = int(condition)
-            return value == compare_value
+        if condition[0:2] in ["==", "!=", ">=", "=>", "<=", "=<"]:
+            return compare_value_value(value, condition[0:2], int(condition[2:]))
+        elif condition[0] in ["=", "<", ">"]:
+            return compare_value_value(value, condition[0], int(condition[1:]))
+        else:
+            return value == int(condition)
 
-        # compare bigger
-        if condition[0] == ">":
-            compare_value = int(condition[1:])
-            return value > compare_value
-
-        # compare less
-        if condition[0] == "<":
-            compare_value = int(condition[1:])
-            return value < compare_value
     except ValueError:
         log.warning(u"Invalid condition {0!s}. Needs to contain an integer.".format(condition))
         return False
@@ -847,11 +840,14 @@ def compare_condition(condition, value):
 def compare_value_value(value1, comparator, value2):
     """
     This function compares value1 and value2 with the comparator.
-    The comparator may be "==", ">" or "<".
+    The comparator may be "==", "=", "!=", ">", "<", ">=", "=>", "<=" or "=<".
     
-    If the values can be converted to integers, they are compared as integers 
+    If the values can be converted to integers or dates, they are compared as such,
     otherwise as strings.
-    
+
+    In case of dates make sure they can be parsed by 'parse_date()', otherwise
+    they will be compared as strings.
+
     :param value1: First value 
     :param value2: Second value
     :param comparator: The comparator
@@ -878,14 +874,47 @@ def compare_value_value(value1, comparator, value2):
         except Exception:
             log.debug("error during date conversion.")
 
-    if comparator == "==":
+    if comparator in ["==", "="]:
         return value1 == value2
     elif comparator == ">":
         return value1 > value2
     elif comparator == "<":
         return value1 < value2
+    elif comparator in ['>=', '=>']:
+        return value1 >= value2
+    elif comparator in ['<=', '=<']:
+        return value1 <= value2
+    elif comparator == '!=':
+        return value1 != value2
+    else:
+        raise Exception("Unknown comparator: {0!s}".format(comparator))
 
-    raise Exception("Unknown comparator: {0!s}".format(comparator))
+
+def compare_generic_condition(cond, key_method, warning):
+    """
+    Compares a condition like "tokeninfoattribute == value".
+    It uses the "key_method" to determine the value of "tokeninfoattribute".
+
+    If the value does not match, it returns False.
+
+    :param cond: A condition containing a comparator like "==", ">", "<"
+    :param key_method: A function call, that get the value from the key
+    :param warning: A warning message to be written to the log file.
+    :return: True of False
+    """
+    key = value = None
+    for comparator in ["==", ">", "<"]:
+        if len(cond.split(comparator)) == 2:
+            key, value = [x.strip() for x in cond.split(comparator)]
+            break
+    if value:
+        res = compare_value_value(key_method(key), comparator, value)
+        log.debug("Comparing {0!s} {1!s} {2!s} with result {3!s}.".format(key, comparator, value, res))
+        return res
+    else:
+        # There is a condition, but we do not know it!
+        log.warning(warning.format(cond))
+        raise Exception("Condition not parsable.")
 
 
 def int_to_hex(serial):
@@ -923,7 +952,7 @@ def parse_legacy_time(ts, return_date=False):
     if not d.tzinfo:
         # we need to reparse the string
         d = parse_date_string(ts,
-                              dayfirst=re.match("^\d\d[/\.]",ts)).replace(
+                              dayfirst=re.match(r"^\d\d[/\.]", ts)).replace(
                                   tzinfo=tzlocal())
     if return_date:
         return d
@@ -1047,20 +1076,12 @@ def censor_connect_string(connect_string):
     """
     Take a SQLAlchemy connect string and return a sanitized version
     that can be written to the log without disclosing the password.
-    The password is replaced with "xxxx".
+    The password is replaced with "***".
     In case any error occurs, return "<error when censoring connect string>"
     """
     try:
-        parsed = urlparse(connect_string)
-        if parsed.password is not None:
-            # We need to censor the ``netloc`` attribute: user:pass@host
-            _, host = parsed.netloc.rsplit("@", 1)
-            new_netloc = u'{}:{}@{}'.format(parsed.username, 'xxxx', host)
-            # Convert the URL to six components. netloc is component #1.
-            splitted = list(parsed)
-            splitted[1] = new_netloc
-            return urlunparse(splitted)
-        return connect_string
+        parsed = sqlalchemy.engine.url.make_url(connect_string)
+        return parsed.__repr__()
     except Exception:
         return "<error when censoring connect string>"
 
@@ -1109,7 +1130,54 @@ def truncate_comma_list(data, max_len):
     return ",".join(data)
 
 
-def check_pin_policy(pin, policy):
+def generate_charlists_from_pin_policy(policy):
+    """
+    This function uses the pin content policy string (e.g. "+cns", "[asdf]") to create the character lists
+    for password generation.
+
+    :param policy: The policy that describes the allowed contents of the PIN (see check_pin_contents)
+    :return: Dictionary with keys "base" for the base set of allowed characters and "requirements"
+     which denotes a list of characters from each of which at least one must be contained in the pin.
+    """
+
+    # regexp to check for pin content policy string validity
+    VALID_POLICY_REGEXP = re.compile(r'^[+-]*[cns]+$|^\[.*\]+$')
+
+    # default: full character list
+    base_characters = "".join(CHARLIST_CONTENTPOLICY.values())
+    # list of strings where a character of each string is required for the pin
+    requirements = []
+
+    if not re.match(VALID_POLICY_REGEXP, policy):
+        raise PolicyError("Unknown character specifier in PIN policy.")
+
+    if policy[0] == "+":
+        # grouping
+        for char in policy[1:]:
+            requirements.append(CHARLIST_CONTENTPOLICY.get(char))
+        requirements = ["".join(requirements)]
+
+    elif policy[0] == "-":
+        # exclusion
+        base_charlist = []
+        for key in CHARLIST_CONTENTPOLICY.keys():
+            if key not in policy[1:]:
+                base_charlist.append(CHARLIST_CONTENTPOLICY[key])
+        base_characters = "".join(base_charlist)
+
+    elif policy[0] == "[" and policy[-1] == "]":
+        # only allowed characters
+        base_characters = policy[1:-1]
+
+    else:
+        for c in policy:
+            if c in CHARLIST_CONTENTPOLICY:
+                requirements.append(CHARLIST_CONTENTPOLICY.get(c))
+
+    return {"base": base_characters, "requirements": requirements}
+
+
+def check_pin_contents(pin, policy):
     """
     The policy to check a PIN can contain of "c", "n" and "s".
     "cn" means, that the PIN should contain a character and a number.
@@ -1121,52 +1189,27 @@ def check_pin_policy(pin, policy):
     :param policy: The policy that describes the allowed contents of the PIN.
     :return: Tuple of True or False and a description
     """
-    chars = {"c": r"[a-zA-Z]",
-             "n": r"[0-9]",
-             "s": r"[\[\].:,;_<>+*!/()=?$§%&#~^-]"}
+
     ret = True
     comment = []
 
     if not policy:
         return False, "No policy given."
 
-    if policy[0] in ["+", "-"] or policy[0] is not "[":
-        for char in policy[1:]:
-            if char not in chars.keys():
-                raise PolicyError("Unknown character specifier in PIN policy.")
+    charlists_dict = generate_charlists_from_pin_policy(policy)
 
-    if policy[0] == "+":
-        # grouping
-        necessary = []
-        for char in policy[1:]:
-            necessary.append(chars.get(char))
-        necessary = "|".join(necessary)
-        if not re.search(necessary, pin):
+    # check for not allowed characters
+    for char in pin:
+        if not char in charlists_dict["base"]:
             ret = False
-            comment.append("Missing character in PIN: {0!s}".format(necessary))
+    if not ret:
+        comment.append("Not allowed character in PIN!")
 
-    elif policy[0] == "-":
-        # exclusion
-        not_allowed = []
-        for char in policy[1:]:
-            not_allowed.append(chars.get(char))
-        not_allowed = "|".join(not_allowed)
-        if re.search(not_allowed, pin):
+    # check requirements
+    for str in charlists_dict["requirements"]:
+        if not re.search(re.compile('[' + re.escape(str) + ']'), pin):
             ret = False
-            comment.append("Not allowed character in PIN!")
-
-    elif policy[0] == "[" and policy[-1] == "]":
-        # only allowed characters
-        allowed_chars = policy[1:-1]
-        for ch in pin:
-            if ch not in allowed_chars:
-                ret = False
-                comment.append("Not allowed character in PIN!")
-    else:
-        for c in chars:
-            if c in policy and not re.search(chars[c], pin):
-                ret = False
-                comment.append("Missing character in PIN: {0!s}".format(chars[c]))
+            comment.append("Missing character in PIN: {0!s}".format(str))
 
     return ret, ",".join(comment)
 
@@ -1208,7 +1251,7 @@ def get_version_number():
     """
     version = "unknown"
     try:
-        version = pkg_resources.get_distribution("privacyidea").version
+        version = importlib_metadata.version("privacyidea")
     except:
         log.info("We are not able to determine the privacyidea version number.")
     return version
@@ -1331,7 +1374,7 @@ def create_tag_dict(logged_in_user=None,
     if escape_html:
         escaped_tags = {}
         for key, value in tags.items():
-            escaped_tags[key] = cgi.escape(value) if value is not None else None
+            escaped_tags[key] = html.escape(value) if value is not None else None
         tags = escaped_tags
 
     return tags
@@ -1382,3 +1425,45 @@ def determine_logged_in_userparams(logged_in_user, params):
         raise PolicyError(u"Unknown role: {}".format(role))
 
     return role, username, realm, admin_user, admin_realm
+
+
+def to_list(input):
+    """
+    Returns a list if either a list, a set or a single string is given.
+    If a single string is given, then it returns a list with this one element.
+
+    :param input: Can be a list a set or a string
+    :return: list of elements
+    """
+    if isinstance(input, list):
+        return input
+    if isinstance(input, set):
+        return list(input)
+    return [input]
+
+
+def parse_string_to_dict(s, split_char=":"):
+    """
+    This function can parse a string that is formatted like:
+
+       :key1: valueA valueB valueC :key2: valueD valueE
+
+    and return a dict:
+
+       {"key1": ["valueA", "valueB", "valueC"],
+        "key2": ["valueD", "valueE"]
+
+    Note: a whitespace is in the string is separating the values.
+    Thus values can not contain a whitespace.
+
+    :param s: The string that should be parsed
+    :param split_char: The character used for splitting the string
+    :return: the dict
+    """
+    # create a list like ["key1", "valueA valueB valueC", "key2", "valueD valueE"]
+    packed_list = [x.strip() for x in s.strip().split(split_char) if x]
+    keys = packed_list[::2]
+    # create a list of the values: [['valueA', 'valueB', 'valueC'], ['valueD', 'valueE']]
+    values = [[x for x in y.split()] for y in packed_list[1::2]]
+    d = {a: b for a, b in zip(keys, values)}
+    return d
